#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include "SDL/SDL.h"
#include "SDL/SDL_image.h"
#include "SDL/SDL_ttf.h"
#include "brikx.h"

SDL_Surface *LoadImage(char *fname,int Rkey,int Gkey,int Bkey)
{
    SDL_Surface *loaded,*optimized=NULL;
    if (loaded=IMG_Load(fname))
      {
        if ((optimized=SDL_DisplayFormat(loaded)) && Rkey>-1)
            SDL_SetColorKey(optimized,SDL_RLEACCEL | SDL_SRCCOLORKEY,SDL_MapRGB(optimized->format,Rkey,Gkey,Bkey));
        if (!optimized) return loaded;
        SDL_FreeSurface(loaded);
      }
    else return NULL;
    return optimized;
}

void PutSurface(SDL_Surface *src,SDL_Surface *dest,SDL_Rect pos,int clipx,int clipy,int width,int height)
{
     if (!src || !dest) return;
     SDL_Rect clip;
     clip.x=clipx;
     clip.y=clipy;
     clip.w=width;
     clip.h=height;
     SDL_BlitSurface(src,clipx==-1?NULL:&clip,dest,&pos);
}

void PutSurface(SDL_Surface *src,SDL_Surface *dest,int x,int y,int clipx,int clipy,int width,int height)
{
     SDL_Rect pos;
     pos.x=x;
     pos.y=y;
     PutSurface(src,dest,pos,clipx,clipy,width,height);
}

int LoadLevel(char *fname,level &t)
{
      FILE *f=fopen(fname,"r");
      if (!f) return 0;
      char s[140];
      int i,j,timelimit;
      fgets(s,130,f);
      timelimit=atoi(s);
      for (i=0; i<=7 && !feof(f); i++)
        {
          fgets(s,130,f);
          for (j=0; j<=7; j++)
            t[j][i]=(s[j*2]-'0')*10+s[j*2+1]-'0';
        }
      fclose(f);
      return timelimit;
}

int SaveLevel(char *fname,level t,int timelimit)
{
    FILE *f=fopen(fname,"w+");
    if (!f) return 0;
    int i,j;
    fprintf(f,"%d\n",timelimit);
    for (i=0; i<=7; i++)
      {
        for (j=0; j<=7; j++)
          fprintf(f,"%02d",t[j][i]);
        fprintf(f,"\n");
      }
    fclose(f);
    return 1;
}

int Input(SDL_Surface *screen,TTF_Font *font,SDL_Color color,int x,int y,char *s,int maxlen)
{
    SDL_Surface *text=NULL,*save=NULL;
    SDL_Event event; 
    bool end=false;
    char t[maxlen+1],ss[strlen(s)+maxlen+2];
    *t=0;
    Uint32 rmask,gmask,bmask,amask;
       #if SDL_BYTEORDER == SDL_BIG_ENDIAN
           rmask = 0xff000000;
           gmask = 0x00ff0000;
           bmask = 0x0000ff00;
           amask = 0x000000ff;
       #else
           rmask = 0x000000ff;
           gmask = 0x0000ff00;
           bmask = 0x00ff0000;
           amask = 0xff000000;
       #endif
    if (!(save=SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_RLEACCEL,WIDTH,HEIGHT,BPP,rmask,gmask,bmask,amask))) return 0;
    SDL_LockSurface(save);
    SDL_LockSurface(screen);
    memcpy(save->pixels,screen->pixels,WIDTH*HEIGHT*BPP/8);
    SDL_UnlockSurface(save);
    SDL_UnlockSurface(screen);
    while (!end)
      {
        SDL_Delay(DELAY);
        sprintf(ss,"%s%s.",s,t);
        if (text) SDL_FreeSurface(text);
        text=TTF_RenderText_Solid(font,ss,color);
        SDL_LockSurface(save);
        SDL_LockSurface(screen);
        memcpy(screen->pixels,save->pixels,WIDTH*HEIGHT*BPP/8);
        SDL_UnlockSurface(save);
        SDL_UnlockSurface(screen);
        PutSurface(text,screen,x,y);
        SDL_Flip(screen);
        while (SDL_PollEvent(&event))
          if (event.type==SDL_KEYDOWN)
	     switch (event.key.keysym.sym)
	       {
		      case SDLK_BACKSPACE:
		        if (*t) t[strlen(t)-1]=0;
		      break;
		      case SDLK_ESCAPE:
			       *t=0;
		      case SDLK_RETURN:
			       end=true;
		      break;
		 default:
		      if (isalnum(event.key.keysym.sym) && strlen(t)<maxlen) sprintf(t,"%s%c",t,event.key.keysym.sym);
	       }
    }
    SDL_FreeSurface(text);
    SDL_FreeSurface(save);
    strcpy(s,t);
    return strlen(t);
}

int SaveHigh(long score,char *name)
{
    int i,j;
    high h;
    char s[NAME_MAXLEN+2];
    FILE *f=fopen(FILE_HIGH,"r");
    if (f)
      {
        for (i=0; !feof(f); i++)
          {
            fgets(s,NAME_MAXLEN,f);
            h[i].score=atoi(s);
            fgets(h[i].name,NAME_MAXLEN,f);
            h[i].name[strlen(h[i].name)-1]=s[strlen(s)-1]=0;
          }
        fclose(f);
        while (i<MAX_HIGHENTRIES)
          {
            h[i].score=h[i].name[0]=0;
            i++;
          }
      }
    else
      for (i=0; i<MAX_HIGHENTRIES; i++)
        h[i].score=h[i].name[0]=0;
    for (i=0; i<MAX_HIGHENTRIES && score<h[i].score; i++)
      ;
    if (i<MAX_HIGHENTRIES)
      {
        for (j=MAX_HIGHENTRIES-1; j>i; j--)
          {
            h[j].score=h[j-1].score;
            strcpy(h[j].name,h[j-1].name);
          }
        h[i].score=score;
        strcpy(h[i].name,name);
        if (!(f=fopen(FILE_HIGH,"w+"))) return -1;
        for (j=0; j<MAX_HIGHENTRIES; j++)
          fprintf(f,"%d\n%s\n",h[j].score,h[j].name);
        fclose(f);
      }
    return i;
}

int SaveGame(level table,long score,int levnum,long timer,int timelimit,char *name)
{
    char s[1024];
    sprintf(s,"%s%s.save",DIR_SAVE,name);
    FILE *f=fopen(s,"w+");
    int i,j;
    if (!f) return -1;
    fprintf(f,"%d\n%ld\n%ld\n%d\n",levnum,score,timer,timelimit);
    for (i=0; i<=7; i++)
      {
        for (j=0; j<=7; j++)
          fprintf(f,"%02d",table[j][i]);
        fprintf(f,"\n");
      }
    fclose(f);   
    return 0;
}

int LoadGame(level &table,long &score,int &levnum,long &timer,int &timelimit,char *name)
{
    char s[1024];
    sprintf(s,"%s%s.save",DIR_SAVE,name);
    FILE *f=fopen(s,"r");
    int i,j;
    if (!f) return -1;
    fgets(s,130,f);
    levnum=atoi(s);
    fgets(s,130,f);
    score=atoi(s);
    fgets(s,130,f);
    timer=atoi(s);
    fgets(s,130,f);
    timelimit=atoi(s);
    for (i=0; i<=7 && !feof(f); i++)
      {
        fgets(s,130,f);
        for (j=0; j<=7; j++)
          table[j][i]=(s[j*2]-'0')*10+s[j*2+1]-'0';
      }
    fclose(f);
    return 0;
}
